/*
DBAS 1100-700
Project 2 DDL
Ryan Brownell, W0274281
*/
--DROP ALL TABLES
DROP TABLE Order_Details;
DROP TABLE Orders;
DROP TABLE Shippers;
DROP TABLE Customers;
DROP TABLE EmployeeTerritories;
DROP TABLE Employees;
DROP TABLE Territories;
DROP TABLE Region;
DROP TABLE WarehouseStock;
DROP TABLE Products;
DROP TABLE Categories;
DROP TABLE Suppliers;
DROP TABLE Warehouses;
DROP TABLE Countries;

--DROP ALL SEQUENCES
DROP SEQUENCE sq_CountryID;
DROP SEQUENCE sq_WarehouseID;
DROP SEQUENCE sq_CustomerID;

--CREATE ALL SEQUENCES
CREATE SEQUENCE sq_CountryID;
CREATE SEQUENCE sq_WarehouseID;
CREATE SEQUENCE sq_CustomerID;

--CREATE ALL TABLES
--CREATE COUNTRIES
CREATE TABLE Countries (
	CountryID int PRIMARY KEY,
	"Name" VARCHAR(15) NOT NULL
);

--CREATE WAREHOUSES
CREATE TABLE Warehouses (
	WarehouseID int PRIMARY KEY,
	"Name" VARCHAR(10) NOT NULL,
	Address VARCHAR(60),
	City VARCHAR(15),
	Region VARCHAR(15),
	PostalCode VARCHAR(10),
	Country int REFERENCES Countries(CountryID)
);

--CREATE SUPPLIERS
CREATE TABLE Suppliers (
	SupplierID int PRIMARY KEY,
	CompanyName VARCHAR(40) NOT NULL,
	ContactName VARCHAR(30),
	ContactTitle VARCHAR(30),
	Address VARCHAR(60),
	City VARCHAR(15),
	Region VARCHAR(15),
	PostalCode VARCHAR(10),
	Country int REFERENCES Countries(CountryID),
	Phone VARCHAR(24),
	Fax VARCHAR(24),
	HomePage VARCHAR(2000)
);

--CREATE CATEGORIES
CREATE TABLE Categories (
	CategoryID int PRIMARY KEY,
	CategoryName VARCHAR(15) NOT NULL,
	Description VARCHAR(255)
);

--CREATE PRODUCTS
CREATE TABLE Products (
	ProductID int PRIMARY KEY,
	ProductName VARCHAR(40) NOT NULL,
	SupplierID int REFERENCES Suppliers(SupplierID),
	CategoryID int REFERENCES Categories(CategoryID),
	QuantityPerUnit VARCHAR(20),
	UnitPrice NUMBER(*, 2),
	Discontinued NUMBER(1) CHECK (Discontinued in (0, 1))
);

--CREATE WAREHOUSE STOCK
CREATE TABLE WarehouseStock (
	WarehouseID int REFERENCES Warehouses(WarehouseID),
	ProductID int REFERENCES Products(ProductID),
	UnitsinStock int,
	UnitsOnOrder int,
	ReorderLevel int,
	CONSTRAINT pk_WarehouseStock PRIMARY KEY (WarehouseID, ProductID)
);

--CREATE REGION
CREATE TABLE Region (
	RegionID int PRIMARY KEY,
	RegionDescription VARCHAR(50) NOT NULL
);

--CREATE TERRITORIES
CREATE TABLE Territories (
	TerritoryID int PRIMARY KEY,
	TerritoryDescription VARCHAR(50) NOT NULL,
	RegionID int NOT NULL REFERENCES Region(RegionID)
);

--CREATE EMPLOYEES
CREATE TABLE Employees (
	EmployeeID int PRIMARY KEY,
	LastName VARCHAR(20) NOT NULL,
	FirstName VARCHAR (10) NOT NULL,
	Title VARCHAR(30),
	TitleOfCourtesy VARCHAR(25),
	FullName VARCHAR(57),
	BirthDate DATE,
	HireDate DATE,
	Address VARCHAR(60),
	City VARCHAR(15),
	Region VARCHAR(15),
	PostalCode VARCHAR(10),
	Country int REFERENCES Countries(CountryID),
	HomePhone VARCHAR(24),
	Extension VARCHAR(4),
	Notes VARCHAR(500),
	ReportsTo int REFERENCES Employees(EmployeeID)
);

--CREATE EMPLOYEE TERRITORIES
CREATE TABLE EmployeeTerritories (
	EmployeeID int REFERENCES Employees(EmployeeID),
	TerritoryID int REFERENCES Territories(TerritoryID),
	CONSTRAINT pk_EmployeeTerritories PRIMARY KEY (EmployeeID, TerritoryID)
);

--CREATE CUSTOMERS
CREATE TABLE Customers (
	CustomerID int PRIMARY KEY,
	CustomerCode VARCHAR(5) NOT NULL,
	CompanyName VARCHAR(40) NOT NULL,
	ContactName VARCHAR(30),
	Address VARCHAR(60),
	City VARCHAR(15),
	Region VARCHAR(15),
	PostalCode VARCHAR(10),
	Country int REFERENCES Countries(CountryID),
	Phone VARCHAR(24),
	Fax VARCHAR(24),
  CONSTRAINT CustomerCode_Unique UNIQUE (CustomerCode) --Customers are uniquely identified by this code.
);

--CREATE SHIPPERS
CREATE TABLE Shippers (
	ShipperID int PRIMARY KEY,
	CompanyName VARCHAR(40) NOT NULL,
	Phone VARCHAR(24)
);

--CREATE ORDERS
CREATE TABLE Orders (
	OrderID int PRIMARY KEY,
	CustomerID int REFERENCES Customers(CustomerID),
	EmployeeID int REFERENCES Employees(EmployeeID),
	OrderDate DATE,
	RequiredDate DATE,
	ShippedDate DATE,
	ShipVia int REFERENCES Shippers(ShipperID),
	Freight number(*,2),
	ShipName VARCHAR(40),
	ShipAddress VARCHAR(60),
	ShipCity VARCHAR(15),
	ShipRegion VARCHAR(15),
	ShipPostalCode VARCHAR(10),
	ShipCountry int REFERENCES Countries(CountryID)
--	ShipperID int --ITEM EXISTS IN THE ORIGINAL ERD (As a separate field from ShipVia) BUT NOT IN THE ORIGINAL DDL. REMOVED AFTER ASKING GEOFF.
);

--CREATE ORDER DETAILS
CREATE TABLE Order_Details (
	OrderID int REFERENCES Orders(OrderID),
	ProductID int REFERENCES Products(ProductID),
	UnitPrice NUMBER(*, 2) NOT NULL,
	Quantity int NOT NULL,
	Discount  NUMBER(*, 2) NOT NULL,
	CONSTRAINT pk_Order_Details PRIMARY KEY (OrderID, ProductID)
);