/*
DBAS 1100-700
Project 2 DML
Ryan Brownell, W0274281
*/
--DELETE TABLE CONTENTS
DELETE FROM Order_Details;
DELETE FROM Orders;
DELETE FROM Shippers;
DELETE FROM Customers;
DELETE FROM EmployeeTerritories;
DELETE FROM Employees;
DELETE FROM Territories; 
DELETE FROM Region;
DELETE FROM WarehouseStock;
DELETE FROM Products;
DELETE FROM Categories;
DELETE FROM Suppliers;
DELETE FROM Warehouses;
DELETE FROM Countries;

--RESET SEQUENCES
--DROP ALL SEQUENCES
DROP SEQUENCE northwind_enhanceddb.sq_CountryID;
DROP SEQUENCE northwind_enhanceddb.sq_WarehouseID;
DROP SEQUENCE northwind_enhanceddb.sq_CustomerID;

--CREATE ALL SEQUENCES
CREATE SEQUENCE northwind_enhanceddb.sq_CountryID;
CREATE SEQUENCE northwind_enhanceddb.sq_WarehouseID;
CREATE SEQUENCE northwind_enhanceddb.sq_CustomerID;

--POPULATE COUNTRY TABLE FROM EXISTING COUNTRY INFORMATION
INSERT INTO northwind_enhanceddb.Countries (
SELECT northwind_enhanceddb.sq_countryID.NEXTVAL, Country
FROM(
SELECT DISTINCT Country
FROM northwind_originaldb.Employees
UNION
SELECT DISTINCT Country
FROM northwind_originaldb.Customers
UNION
SELECT DISTINCT Country
FROM northwind_originaldb.Suppliers
UNION
SELECT DISTINCT ShipCountry AS Country
FROM northwind_originaldb.Orders
ORDER BY Country))
;

--POPULATE WAREHOUSES TABLE
--(Populating fake addresses as per instructions provided by email.)
INSERT INTO northwind_enhanceddb.Warehouses VALUES(northwind_enhanceddb.sq_WarehouseID.NEXTVAL, 'Dockside', '123 Water Street', 'Faketown', 'Acadia', 'F4K 3A9', (SELECT northwind_enhanceddb.Countries.CountryID FROM northwind_enhanceddb.Countries WHERE "Name" = 'Canada' ));
INSERT INTO northwind_enhanceddb.Warehouses VALUES(northwind_enhanceddb.sq_WarehouseID.NEXTVAL, 'Airport', '123 Air Road', 'Faketown', 'Acadia', 'F4K 3P7', (SELECT northwind_enhanceddb.Countries.CountryID FROM northwind_enhanceddb.Countries WHERE "Name" = 'Canada' ));
INSERT INTO northwind_enhanceddb.Warehouses Values(northwind_enhanceddb.sq_WarehouseID.NEXTVAL, 'Central', '123 Central Avenue', 'Faketown', 'Acadia', 'F4K 3R0', (SELECT northwind_enhanceddb.Countries.CountryID FROM northwind_enhanceddb.Countries WHERE "Name" = 'Canada' ));

--MIGRATE DATA
--MIGRATE SUPPLIERS TABLE
INSERT INTO northwind_enhanceddb.Suppliers (
SELECT northwind_originaldb.Suppliers.SupplierID, northwind_originaldb.Suppliers.CompanyName, northwind_originaldb.Suppliers.ContactName,
northwind_originaldb.Suppliers.ContactTitle, northwind_originaldb.Suppliers.Address, northwind_originaldb.Suppliers.City, northwind_originaldb.Suppliers.Region, northwind_originaldb.Suppliers.PostalCode,
northwind_enhanceddb.Countries.CountryID AS COUNTRY, northwind_originaldb.Suppliers.Phone, northwind_originaldb.Suppliers.Fax, northwind_originaldb.Suppliers.HomePage
FROM northwind_originaldb.Suppliers
JOIN northwind_enhanceddb.Countries ON northwind_enhanceddb.Countries."Name" = northwind_originaldb.Suppliers.Country);

--MIGRATE CATEGORIES TABLE
INSERT INTO northwind_enhanceddb.Categories(
SELECT northwind_originaldb.Categories.CategoryID, northwind_originaldb.Categories.CategoryName, northwind_originaldb.Categories.Description
FROM northwind_originaldb.Categories);

--MIGRATE PRODUCTS TABLE
INSERT INTO northwind_enhanceddb.Products(
SELECT northwind_originaldb.products.productid, northwind_originaldb.products.ProductName, northwind_originaldb.products.SupplierID,
northwind_originaldb.products.CategoryID, northwind_originaldb.products.QuantityPerUnit, northwind_originaldb.products.UnitPrice, northwind_originaldb.products.Discontinued
FROM northwind_originaldb.products);
 
--POPULATE AND MIGRATE WAREHOUSE STOCK TABLE
--ASSIGNING ALL STOCK TO THE CENTRAL WAREHOUSE FOR NOW. MANUAL ADJUSTMENT WILL BE REQUIRED AND COMPLETED BY INDIVIDUALS WHO HAVE, OR HAVE ACCESS TO KNOWLEDGE OF STOCK LOCATION.
INSERT INTO northwind_enhanceddb.WarehouseStock(
SELECT northwind_enhanceddb.sq_WarehouseID.CURRVAL, northwind_enhanceddb.products.productid, northwind_originaldb.products.UnitsInStock,
northwind_originaldb.products.UnitsOnOrder, northwind_originaldb.products.ReorderLevel
FROM northwind_enhanceddb.products
JOIN NORTHWIND_ORIGINALDB.PRODUCTS ON northwind_enhanceddb.products.productid = northwind_originaldb.products.productid);

--MIGRATE CUSTOMERS TABLE
INSERT INTO northwind_enhanceddb.Customers(
SELECT northwind_enhanceddb.sq_CustomerID.NEXTVAL, northwind_originaldb.Customers.Customerid AS CUSTOMERCODE, northwind_originaldb.Customers.CompanyName, northwind_originaldb.Customers.ContactName,
northwind_originaldb.Customers.Address, northwind_originaldb.Customers.City, northwind_originaldb.Customers.Region, northwind_originaldb.Customers.PostalCode,northwind_enhanceddb.Countries.CountryID AS COUNTRY,
northwind_originaldb.Customers.Phone, northwind_originaldb.Customers.Fax
FROM northwind_originaldb.Customers
JOIN northwind_enhanceddb.Countries ON northwind_enhanceddb.Countries."Name" = northwind_originaldb.customers.Country);

--MIGRATE REGION TABLE
--(Includes legacy primary key field to be removed later)
INSERT INTO NORTHWIND_ENHANCEDDB.Region (
SELECT northwind_originaldb.Region.RegionID, northwind_originaldb.Region.RegionDescription
FROM northwind_originaldb.Region);

--MIGRATE TERRITORIES
INSERT INTO northwind_enhanceddb.Territories (
SELECT northwind_originaldb.territories.TerritoryID, northwind_originaldb.territories.TerritoryDescription, northwind_originaldb.territories.regionID
FROM northwind_originaldb.territories);

--MIGRATE EMPLOYEES
INSERT INTO northwind_enhanceddb.Employees(
SELECT northwind_originaldb.Employees.EmployeeID, northwind_originaldb.Employees.LastName, northwind_originaldb.Employees.FirstName, northwind_originaldb.Employees.Title,
northwind_originaldb.Employees.titleofcourtesy,
NVL2(northwind_originaldb.Employees.titleofcourtesy, northwind_originaldb.Employees.titleofcourtesy || ' ', '')|| northwind_originaldb.Employees.FirstName || ' ' || northwind_originaldb.Employees.LastName AS FullName,
northwind_originaldb.Employees.BirthDate, northwind_originaldb.Employees.HireDate, northwind_originaldb.Employees.Address, northwind_originaldb.Employees.City, northwind_originaldb.Employees.Region,
northwind_originaldb.Employees.PostalCode, northwind_enhanceddb.Countries.CountryID, northwind_originaldb.Employees.HomePhone, northwind_originaldb.Employees.Extension,
northwind_originaldb.Employees.Notes, northwind_originaldb.Employees.ReportsTo
FROM northwind_originaldb.employees
JOIN northwind_enhanceddb.Countries ON northwind_enhanceddb.countries."Name" = northwind_originaldb.employees.Country);

--MIGRATE EMPLOYEE TERRITORIES
INSERT INTO northwind_enhanceddb.employeeterritories (
SELECT northwind_originaldb.employeeterritories.employeeid, northwind_originaldb.employeeterritories.territoryid
FROM northwind_originaldb.employeeterritories);

--MIGRATE SHIPPERS
--(Includes legacy primary key field to be removed later)
INSERT INTO northwind_enhanceddb.Shippers(
SELECT northwind_originaldb.Shippers.ShipperID, northwind_originaldb.Shippers.CompanyName, northwind_originaldb.Shippers.Phone
FROM northwind_originaldb.Shippers);

--MIGRATE ORDERS
INSERT INTO ORDERS(
SELECT northwind_originaldb.orders.orderid, northwind_enhanceddb.customers.customerid, northwind_originaldb.orders.employeeid AS EmployeeID,
TO_DATE(northwind_originaldb.orders.OrderDate, 'mm/dd/yyyy'), TO_DATE(northwind_originaldb.orders.RequiredDate, 'mm/dd/yyyy'), TO_DATE(northwind_originaldb.orders.ShippedDate, 'mm/dd/yyyy'),
northwind_originaldb.orders.shipvia AS ShipVia, northwind_originaldb.orders.Freight, northwind_originaldb.orders.ShipName, northwind_originaldb.orders.ShipAddress, northwind_originaldb.orders.ShipCity,
northwind_originaldb.orders.ShipRegion, northwind_originaldb.orders.ShipPostalCode, northwind_enhanceddb.Countries.CountryID AS SHIPCOUNTRY
FROM northwind_originaldb.Orders
JOIN northwind_enhanceddb.customers ON northwind_enhanceddb.customers.customercode = northwind_originaldb.orders.customerid
JOIN northwind_enhanceddb.countries ON northwind_enhanceddb.countries."Name" = northwind_originaldb.orders.shipcountry);

--MIGRATE ORDER_DETAILS
INSERT INTO ORDER_DETAILS(
SELECT northwind_originaldb.order_details.orderid, northwind_originaldb.order_details.productid, northwind_originaldb.Order_Details.UnitPrice, northwind_originaldb.Order_Details.Quantity,
northwind_originaldb.Order_Details.Discount
FROM northwind_originaldb.order_details);