/*
DBAS 1100-700
Project 2 TEST PLAN
Ryan Brownell, W0274281

NOTE: TESTS ON FIELDS THAT HAVE NULL RECORDS WERE ADJUSTED TO ENSURE PROPER COMPARISON
THIS METHOD WAS NOT APPLIED TO ALL FIELDS THAT CAN BE NULL BUT RATHER ONLY ONES THAT 
CONTAINED AT LEAST ONE NULL DATA VALUE. SHOULD THE FINAL MIGRATION USE A DIFFERENT
DATABASE THAN THE ONE PROVIDED, THE ADJUSTMENTS SHOULD BE APPLIED TO ALL FIELDS
THAT DO NOT HAVE A NOT NULL CONSTRAINT.
*/

--Verify successful creation of all database elements.
--THIS QUERY WILL RESULT IN THE DISPLAY OF ROWS THAT DO NOT MATCH ROWS IN THE RESPECTIVE COUNTERPART TABLE.
--A SUCCESSFUL TEST WILL DISPLAY THE FOLLOWING NEW TABLES THAT DO NOT EXIST IN THE ORIGINAL DATABASE:
--WAREHOUSES, WAREHOUSESTOCK, and COUNTRIES.

SELECT *
FROM (SELECT * FROM ALL_TABLES t1 WHERE OWNER = 'NORTHWIND_ENHANCEDDB') t1
FULL OUTER JOIN ( SELECT * FROM ALL_TABLES t1 WHERE OWNER = 'NORTHWIND_ORIGINALDB') t2 ON t1.table_name = t2.table_name
WHERE t1.table_name IS NULL or t2.table_name IS NULL;

--RECORD COUNT
--THE FOLLOWING TESTS VERIFY THE COUNTS OF ALL RECORDS IN BOTH THE OLD AND NEW DATABASES.
--ALL OF THESE TESTS ASSUME THAT NO ADDITIONAL DATA WAS ADDED TO EITHER DATABASE WITHOUT ALSO BEING ADDED TO THE OTHER.

--Verify ORDER_DETAILS Record Count
--A SUCCESSFUL TEST WILL DISPLAY ONE ROW WITH ALL COLUMNS EQUAL TO THE SAME VALUE
SELECT *
FROM (SELECT COUNT(*) AS ORIG_ORDER_DETAIL
  FROM NORTHWIND_ORIGINALDB.ORDER_DETAILS),
  (SELECT COUNT(*) AS ENHANCED_ORDER_DETAIL
  FROM NORTHWIND_ENHANCEDDB.ORDER_DETAILS);

--Verify ORDERS Record Count
--A SUCCESSFUL TEST WILL DISPLAY ONE ROW WITH ALL COLUMNS EQUAL TO THE SAME VALUE
SELECT *
FROM (SELECT COUNT(ORDERID) AS ORIG_ORDERS
  FROM NORTHWIND_ORIGINALDB.ORDERS),
  (SELECT COUNT(ORDERID) AS ENHANCED_ORDERS
  FROM NORTHWIND_ENHANCEDDB.ORDERS);

--Verify SHIPPERS Record Count
--A SUCCESSFUL TEST WILL DISPLAY ONE ROW WITH ALL COLUMNS EQUAL TO THE SAME VALUE
SELECT *
FROM (SELECT COUNT(*) AS ORIG_SHIPPERS
  FROM NORTHWIND_ORIGINALDB.SHIPPERS),
  (SELECT COUNT(*) AS ENHANCED_SHIPPERS
  FROM NORTHWIND_ENHANCEDDB.SHIPPERS);

--Verify CUSTOMERS Record Count
--A SUCCESSFUL TEST WILL DISPLAY ONE ROW WITH ALL COLUMNS EQUAL TO THE SAME VALUE
SELECT *
FROM (SELECT COUNT(*) AS ORIG_CUSTOMERS
  FROM NORTHWIND_ORIGINALDB.CUSTOMERS),
  (SELECT COUNT(*) AS ENHANCED_CUSTOMERS
  FROM NORTHWIND_ENHANCEDDB.CUSTOMERS);

--Verify EMPLOYEETERRITORIES Record Count
--A SUCCESSFUL TEST WILL DISPLAY ONE ROW WITH ALL COLUMNS EQUAL TO THE SAME VALUE
SELECT *
FROM (SELECT COUNT(*) AS ORIG_EMPLOYEETERRITORIES
  FROM NORTHWIND_ORIGINALDB.EMPLOYEETERRITORIES),
  (SELECT COUNT(*) AS ENHANCED_EMPLOYEETERRITORIES
  FROM NORTHWIND_ENHANCEDDB.EMPLOYEETERRITORIES);
  
--Verify EMPLOYEES Record Count
--A SUCCESSFUL TEST WILL DISPLAY ONE ROW WITH ALL COLUMNS EQUAL TO THE SAME VALUE
SELECT *
FROM (SELECT COUNT(*) AS ORIG_EMPLOYEES
  FROM NORTHWIND_ORIGINALDB.EMPLOYEES),
  (SELECT COUNT(*) AS ENHANCED_EMPLOYEES
  FROM NORTHWIND_ENHANCEDDB.EMPLOYEES);
  
--Verify TERRITORIES Record Count
--A SUCCESSFUL TEST WILL DISPLAY ONE ROW WITH ALL COLUMNS EQUAL TO THE SAME VALUE
SELECT *
FROM (SELECT COUNT(*) AS ORIG_TERRITORIES
  FROM NORTHWIND_ORIGINALDB.TERRITORIES),
  (SELECT COUNT(*) AS ENHANCED_TERRITORIES
  FROM NORTHWIND_ENHANCEDDB.TERRITORIES);
  
--Verify REGION Record Count
--A SUCCESSFUL TEST WILL DISPLAY ONE ROW WITH ALL COLUMNS EQUAL TO THE SAME VALUE
SELECT *
FROM (SELECT COUNT(*) AS ORIG_REGION
  FROM NORTHWIND_ORIGINALDB.REGION),
  (SELECT COUNT(*) AS ENHANCED_REGION
  FROM NORTHWIND_ENHANCEDDB.REGION);
  
--Verify WAREHOUSESTOCK Record Count
--A SUCCESSFUL TEST WILL DISPLAY ONE ROW WITH ALL COLUMNS EQUAL TO THE SAME VALUE
SELECT *
FROM (SELECT COUNT(*) AS ORIG_PRODUCTS
  FROM NORTHWIND_ORIGINALDB.PRODUCTS), --COMPARING AGAINST PRODUCTS BECAUSE TABLES IS NEW AND SHOULD REFERENCE EVERY PRODUCT IN THE PRODUCTS TABLE AT LEAST ONCE.
  (SELECT COUNT(*) AS ENHANCED_WAREHOUSESTOCK
  FROM NORTHWIND_ENHANCEDDB.WAREHOUSESTOCK);

--Verify PRODUCTS Record Count
--A SUCCESSFUL TEST WILL DISPLAY ONE ROW WITH ALL COLUMNS EQUAL TO THE SAME VALUE
SELECT *
FROM (SELECT COUNT(*) AS ORIG_PRODUCTS
  FROM NORTHWIND_ORIGINALDB.PRODUCTS),
  (SELECT COUNT(*) AS ENHANCED_PRODUCTS
  FROM NORTHWIND_ENHANCEDDB.PRODUCTS);

--Verify CATEGORIES Record Count
--A SUCCESSFUL TEST WILL DISPLAY ONE ROW WITH ALL COLUMNS EQUAL TO THE SAME VALUE
SELECT *
FROM (SELECT COUNT(*) AS ORIG_CATEGORIES
  FROM NORTHWIND_ORIGINALDB.CATEGORIES),
  (SELECT COUNT(*) AS ENHANCED_CATEGORIES
  FROM NORTHWIND_ENHANCEDDB.CATEGORIES);

--Verify SUPPLIERS Record Count
--A SUCCESSFUL TEST WILL DISPLAY ONE ROW WITH ALL COLUMNS EQUAL TO THE SAME VALUE
SELECT *
FROM (SELECT COUNT(*) AS ORIG_SUPPLIERS
  FROM NORTHWIND_ORIGINALDB.SUPPLIERS),
  (SELECT COUNT(*) AS ENHANCED_SUPPLIERS
  FROM NORTHWIND_ENHANCEDDB.SUPPLIERS);

--Verify WAREHOUSES Record Count
--A SUCCESSFUL TEST WILL DISPLAY ONE ROW AND ONE COLUMN WITH THE NUMBER 3
SELECT *
FROM (SELECT COUNT(*) AS ENHANCED_WAREHOUSES
  FROM NORTHWIND_ENHANCEDDB.WAREHOUSES);

--Verify COUNTRY Record Count
--A SUCCESSFUL TEST WILL DISPLAY ONE ROW WITH ALL COLUMNS EQUAL TO THE SAME VALUE
SELECT *
  FROM (SELECT COUNT(*) AS ORIGINAL_COUNTRIES
    FROM (SELECT Country
    FROM(
    SELECT DISTINCT Country
    FROM northwind_originaldb.Employees
    UNION
    SELECT DISTINCT Country
    FROM northwind_originaldb.Customers
    UNION
    SELECT DISTINCT Country
    FROM northwind_originaldb.Suppliers
    UNION
    SELECT DISTINCT ShipCountry AS Country
    FROM northwind_originaldb.Orders
    ORDER BY Country))), 
  (SELECT COUNT(*) AS ENHANCED_COUNTRIES
  FROM NORTHWIND_ENHANCEDDB.COUNTRIES);


--DATA VERIFICATION
--THE FOLLOWING TESTS VERIFY THAT ALL DATA IN THE OLD DATABASE HAS BEEN MIGRATED TO THE NEW DATABASE.

--Verify ORDER_DETAILS Successful Data Migration
--THIS QUERY WILL RESULT IN THE DISPLAY OF ROWS THAT DO NOT MATCH ROWS IN THE RESPECTIVE COUNTERPART TABLE.
--A SUCCESSFUL TEST WILL DISPLAY AN EMPTY RESULT
SELECT *
FROM northwind_originaldb.order_details t1
FULL OUTER JOIN northwind_enhanceddb.order_details t2 ON t1.orderid = t2.orderid
  AND t1.productid = t2.productid
  AND t1.unitprice = t2.unitprice
  AND t1.quantity = t2.quantity
WHERE (t1.orderID is NULL AND t1.productID is NULL)
  OR (t2.orderID is NULL AND t2.productID is NULL);

--Verify ORDERS Successful Data Migration
--This also verifies that all SHIPCOUNTRY and CUSTOMERID fields point to the correct records in their parent tables.
--THIS QUERY WILL RESULT IN THE DISPLAY OF ROWS THAT DO NOT MATCH ROWS IN THE RESPECTIVE COUNTERPART TABLE.
--A SUCCESSFUL TEST WILL DISPLAY AN EMPTY RESULT
SELECT *
FROM northwind_originaldb.orders t1
FULL OUTER JOIN northwind_enhanceddb.orders t2 ON t1.orderid = t2.orderid
  --CustomerID below...
  AND t1.employeeid = t2.employeeid
  AND TO_DATE(t1.orderdate, 'mm/dd/yyyy') = t2.orderdate
  AND TO_DATE(t1.requireddate, 'mm/dd/yyyy') = t2.requireddate
  AND NVL(TO_DATE(t1.shippeddate, 'mm/dd/yyyy'), TO_DATE('01/01/1900', 'mm/dd/yyyy')) = NVL(t2.shippeddate, TO_DATE('01/01/1900', 'mm/dd/yyyy')) --NEEDED HANDLER FOR NULL BECAUSE 2 NULL FIELDS CAN NOT BE COMPARED. Using dummy date January 1, 1900 as null value.
  AND t1.shipvia = t2.shipvia
  AND t1.freight = t2.freight
  AND t1.shipname = t2.shipname
  AND t1.shipcity = t2.shipcity
  AND NVL(t1.shipregion, '*NULL*') = NVL(t2.shipregion, '*NULL*') --NEEDED HANDLER FOR NULL BECAUSE 2 NULL FIELDS CAN NOT BE COMPARED.
  AND NVL(t1.shipregion, '*NULL*') = NVL(t2.shipregion, '*NULL*') --NEEDED HANDLER FOR NULL BECAUSE 2 NULL FIELDS CAN NOT BE COMPARED.
  AND NVL(t1.shippostalcode, '*NULL*') = NVL(t2.shippostalcode, '*NULL*') --NEEDED HANDLER FOR NULL BECAUSE 2 NULL FIELDS CAN NOT BE COMPARED.
  --Country below...
FULL OUTER JOIN northwind_enhanceddb.countries t3 ON t1.shipcountry = t3."Name" --Cross reference with new Countries table.
  AND t2.shipcountry = t3.countryid
FULL OUTER JOIN northwind_enhanceddb.customers t4 ON t1.customerid = t4.customercode --Cross reference with Customers table to ensure that the referenced customers are the same.
  AND t2.customerid = t4.customerid
WHERE ((t1.orderID IS NULL OR t2.orderID IS NULL) OR (t3.countryid IS NULL OR t2.shipcountry IS NULL) OR (t4.customerid IS NULL OR t2.customerid IS NULL))
  AND (t1.shipcountry IS NOT NULL OR t2.shipcountry IS NOT NULL) --Ensures that countries without an associated order are not shown as part of the join.
  AND (t1.customerid IS NOT NULL OR t2.customerid IS NOT NULL); --Ensures that customers without an associated order as not shown as part of the join.

--Verify SHIPPERS Successful Data Migration
--THIS QUERY WILL RESULT IN THE DISPLAY OF ROWS THAT DO NOT MATCH ROWS IN THE RESPECTIVE COUNTERPART TABLE.
--A SUCCESSFUL TEST WILL DISPLAY AN EMPTY RESULT
SELECT *
FROM northwind_originaldb.shippers t1
FULL OUTER JOIN northwind_enhanceddb.shippers t2 ON t1.shipperid = t2.shipperid
  AND t1.companyname = t2.companyname
  AND t1.phone = t2.phone
WHERE t1.shipperid is NULL OR t2.shipperid is NULL;

--Verify CUSTOMERS Successful Data Migration
--This also verifies that all COUNTRY fields point to the correct records in their parent tables.
--THIS QUERY WILL RESULT IN THE DISPLAY OF ROWS THAT DO NOT MATCH ROWS IN THE RESPECTIVE COUNTERPART TABLE.
--A SUCCESSFUL TEST WILL DISPLAY AN EMPTY RESULT
SELECT *
FROM northwind_originaldb.customers t1
FULL OUTER JOIN northwind_enhanceddb.customers t2 ON t1.customerid = t2.customercode --Values of original customerid moved to enhanced customercode field.
  --Skip t2.customer ID because there is no corresponding data as it only contains newly generated keys.
  AND t1.companyname = t2.companyname
  AND t1.contactname = t2.contactname
  AND t1.address = t2.address
  AND t1.city = t2.city
  AND NVL(t1.region, '*NULL*') = NVL(t2.region, '*NULL*') --NEEDED HANDLER FOR NULL BECAUSE 2 NULL FIELDS CAN NOT BE COMPARED.
  AND NVL(t1.postalcode, '*NULL*') = NVL(t2.postalcode, '*NULL*') --NEEDED HANDLER FOR NULL BECAUSE 2 NULL FIELDS CAN NOT BE COMPARED.
  --Country below...
  AND t1.phone = t2.phone
  AND NVL(t1.fax, '*NULL*') = NVL(t2.fax, '*NULL*') --NEEDED HANDLER FOR NULL BECAUSE 2 NULL FIELDS CAN NOT BE COMPARED.
FULL OUTER JOIN northwind_enhanceddb.countries t3 ON t1.country = t3."Name" --Cross reference with new Countries table.
  AND t2.country = t3.countryid
WHERE ((t1.customerID IS NULL OR t2.customerID IS NULL) OR (t3.countryid IS NULL OR t2.country IS NULL))
  AND (t1.country IS NOT NULL OR t2.country IS NOT NULL); --Ensures that countries without an associated order are not shown as part of the join.

--Verify EMPLOYEETERRITORIES Successful Data Migration
--THIS QUERY WILL RESULT IN THE DISPLAY OF ROWS THAT DO NOT MATCH ROWS IN THE RESPECTIVE COUNTERPART TABLE.
--A SUCCESSFUL TEST WILL DISPLAY AN EMPTY RESULT
SELECT *
FROM northwind_originaldb.employeeterritories t1
FULL OUTER JOIN northwind_enhanceddb.employeeterritories t2 ON t1.employeeid = t2.employeeid
  AND t1.territoryid = t2.territoryid
WHERE (t1.territoryID is NULL AND t1.employeeID is NULL)
  OR (t2.territoryID is NULL AND t2.employeeID is NULL);

--Verify EMPLOYEES Successful Data Migration PART 1 OF 2
--This also verifies that all COUNTRY fields point to the correct records in their parent tables.
--THIS QUERY WILL RESULT IN THE DISPLAY OF ROWS THAT DO NOT MATCH ROWS IN THE RESPECTIVE COUNTERPART TABLE.
--A SUCCESSFUL TEST WILL DISPLAY AN EMPTY RESULT
SELECT *
FROM northwind_originaldb.employees t1
FULL OUTER JOIN northwind_enhanceddb.employees t2 ON t1.employeeid = t2.employeeid
  AND t1.lastname = t2.lastname
  AND t1.firstname = t2.firstname
  AND t1.title = t2.title
  AND NVL(t1.titleofcourtesy, '*NULL*') = NVL(t2.titleofcourtesy, '*NULL*') --NEEDED HANDLER FOR NULL BECAUSE 2 NULL FIELDS CAN NOT BE COMPARED.
  --SKIPPING FULLNAME FOR LATER VERIFICATION STEP
  AND t1.birthdate = t2.birthdate
  AND t1.hiredate = t2.hiredate
  AND t1.address = t2.address
  AND t1.city = t2.city
  AND NVL(t1.region, '*NULL*') = NVL(t2.region, '*NULL*') --NEEDED HANDLER FOR NULL BECAUSE 2 NULL FIELDS CAN NOT BE COMPARED.
  AND t1.postalcode = t2.postalcode
  --Country below...
  AND t1.homephone = t2.homephone
  AND t1.extension = t2.extension
  AND t1.notes = t2.notes
  AND NVL(t1.reportsto, -1) = NVL(t2.reportsto, -1) --NEEDED HANDLER FOR NULL BECAUSE 2 NULL FIELDS CAN NOT BE COMPARED. Used -1 as dummy foreign key.
FULL OUTER JOIN northwind_enhanceddb.countries t3 ON t1.country = t3."Name" --Cross reference with new Countries table.
  AND t2.country = t3.countryid
WHERE ((t1.employeeID IS NULL OR t2.employeeID IS NULL) OR (t3.countryid IS NULL OR t2.country IS NULL))
  AND (t1.country IS NOT NULL OR t2.country IS NOT NULL); --Ensures that countries without an associated order are not shown as part of the join.

--Verify EMPLOYEES Successful Data Migration - FullName Creation PART 2 OF 2
--THIS QUERY WILL RESULT IN THE DISPLAY OF ROWS HAVE NOT CONSTRUCTED THE FULLNAME CORRECTLY.
--A SUCCESSFUL TEST WILL DISPLAY AN EMPTY RESULT
SELECT *
FROM northwind_enhanceddb.employees t1
FULL OUTER JOIN northwind_enhanceddb.employees t2 ON t1.employeeID = t2.employeeID
  AND NVL2(t1.titleofcourtesy, t1.titleofcourtesy || ' ', '')|| t1.FirstName || ' ' || t1.LastName = t2.fullname
WHERE t2.employeeid is NULL;

--Verify TERRITORIES Successful Data Migration
--THIS QUERY WILL RESULT IN THE DISPLAY OF ROWS THAT DO NOT MATCH ROWS IN THE RESPECTIVE COUNTERPART TABLE.
--A SUCCESSFUL TEST WILL DISPLAY AN EMPTY RESULT
SELECT *
FROM northwind_originaldb.territories t1
FULL OUTER JOIN northwind_enhanceddb.territories t2 ON t1.territoryid = t2.territoryid
  AND t1.territorydescription = t2.territorydescription
  AND t1.regionid = t2.regionid
WHERE t1.territoryid is NULL OR t2.territoryid is NULL;

--Verify REGION Successful Data Migration
--THIS QUERY WILL RESULT IN THE DISPLAY OF ROWS THAT DO NOT MATCH ROWS IN THE RESPECTIVE COUNTERPART TABLE.
--A SUCCESSFUL TEST WILL DISPLAY AN EMPTY RESULT
SELECT *
FROM northwind_originaldb.region t1
FULL OUTER JOIN northwind_enhanceddb.region t2 ON t1.regionid = t2.regionid
  AND t1.regiondescription = t2.regiondescription
WHERE t1.regionid IS NULL OR t2.regionid IS NULL;

--Verify WAREHOUSESTOCK Successful Data Migration
--THIS QUERY WILL RESULT IN THE DISPLAY OF ROWS THAT DO NOT MATCH ROWS IN THE RESPECTIVE COUNTERPART TABLE.
--A SUCCESSFUL TEST WILL DISPLAY AN EMPTY RESULT
--NOTE: NO DATA AVAILABLE ON WHICH WAREHOUSE STOCK IS LOCATED. RECORDED ALL STOCK AS BEING IN THE CENTRAL WAREHOUSE (Warehouseid = 3).
SELECT *
FROM northwind_originaldb.Products t1
FULL OUTER JOIN northwind_enhanceddb.WarehouseStock t2 ON t1.PRODUCTID = t2.productid
  AND t1.unitsinstock = t2.unitsinstock
  AND t1.unitsonorder = t2.unitsonorder
  AND t1.reorderlevel = t2.reorderlevel
WHERE (t1.productid IS NULL OR t2.productid IS NULL) AND t2.warehouseid = 3;

--Verify PRODUCTS Successful Data Migration
--THIS QUERY WILL RESULT IN THE DISPLAY OF ROWS THAT DO NOT MATCH ROWS IN THE RESPECTIVE COUNTERPART TABLE.
--A SUCCESSFUL TEST WILL DISPLAY AN EMPTY RESULT
SELECT *
FROM northwind_originaldb.PRODUCTS t1
FULL OUTER JOIN northwind_enhanceddb.PRODUCTS t2 ON t1.productid = t2.productid
  AND t1.productname = t2.productname
  AND t1.supplierid = t2.supplierid
  AND t1.categoryid = t2.categoryid
  AND t1.quantityperunit = t2.quantityperunit
  AND t1.unitprice = t2.unitprice
  --SKIPPING UNITSINSTOCK (ALREADY VERIFIED ABOVE)
  --SKIPPING UNITSONORDER (ALREADY VERIFIED ABOVE)
  --SKIPPING REORDERLEVEL (ALREADY VERIFIED ABOVE)
  AND t1.discontinued = t2.discontinued
WHERE t1.productid is NULL OR t2.productid is NULL;

--Verify CATEGORIES Successful Data Migration
--THIS QUERY WILL RESULT IN THE DISPLAY OF ROWS THAT DO NOT MATCH ROWS IN THE RESPECTIVE COUNTERPART TABLE.
--A SUCCESSFUL TEST WILL DISPLAY AN EMPTY RESULT
SELECT *
FROM northwind_originaldb.categories t1
FULL OUTER JOIN northwind_enhanceddb.categories t2 ON t1.categoryid = t2.categoryid
  AND t1.categoryname = t2.categoryname
  AND t1.description = t2.description
WHERE t1.categoryid is NULL OR t2.categoryid is NULL;

--Verify SUPPLIERS Successful Data Migration
--This also verifies that all COUNTRY fields point to the correct records in their parent tables.
--THIS QUERY WILL RESULT IN THE DISPLAY OF ROWS THAT DO NOT MATCH ROWS IN THE RESPECTIVE COUNTERPART TABLE.
--A SUCCESSFUL TEST WILL DISPLAY AN EMPTY RESULT
SELECT *
FROM northwind_originaldb.suppliers t1
FULL OUTER JOIN northwind_enhanceddb.suppliers t2 ON t1.supplierid = t2.supplierid
  AND t1.companyname = t2.companyname
  AND t1.contactname = t2.contactname
  AND t1.contacttitle = t2.contacttitle
  AND t1.address = t2.address
  AND t1.city = t2.city
  AND NVL(t1.region, '*NULL*') = NVL(t2.region, '*NULL*') --NEEDED HANDLER FOR NULL BECAUSE 2 NULL FIELDS CAN NOT BE COMPARED.
  AND t1.postalcode = t2.postalcode
  --Country below...
  AND t1.phone = t2.phone
  AND NVL(t1.fax, '*NULL*') = NVL(t2.fax, '*NULL*') --NEEDED HANDLER FOR NULL BECAUSE 2 NULL FIELDS CAN NOT BE COMPARED.
  AND NVL(t1.HOMEPAGE, '*NULL*') = NVL(t2.HOMEPAGE, '*NULL*') --NEEDED HANDLER FOR NULL BECAUSE 2 NULL FIELDS CAN NOT BE COMPARED.
FULL OUTER JOIN northwind_enhanceddb.countries t3 ON t1.country = t3."Name" --Cross reference with new Countries table.
  AND t2.country = t3.countryid
WHERE ((t1.supplierID IS NULL OR t2.supplierID IS NULL) OR (t3.countryid IS NULL OR t2.country IS NULL))
  AND (t1.country IS NOT NULL OR t2.country IS NOT NULL); --Ensures that countries without an associated order are not shown as part of the join.
  
--DATA VERIFICATION NOT POSSIBLE FOR WAREHOUSES AS ALL CONTAINING DATA IS NEW... NEEDS TO BE MANUALLY PERFORMED.
--COUNTRIES DATA VERIFICATION NOT NECESSARY AS ALL PREVIOUS VERIFICATIONS INVOLVING THE COUNTRY FIELD PERFORM THE NECESSARY VALIDATION (SEE RECORD COUNT FOR FURTHER VERIFICATION)


--TEST DATA INPUT
--Used negative integers as primary keys to ensure that current valid data is not impacted by tests.

--ADD COUNTRIES
INSERT INTO northwind_enhanceddb.COUNTRIES VALUES(-1, 'Testland');
INSERT INTO northwind_enhanceddb.COUNTRIES VALUES(-2, 'The Examnia');

--ADD WAREHOUSES
INSERT INTO northwind_enhanceddb.WAREHOUSES VALUES(-1, 'Bunker', '321 Not-Here Boulevard', 'Fakeville', 'Seaside', 123456, -1);
INSERT INTO northwind_enhanceddb.WAREHOUSES VALUES(-2, 'OwnersRes', '231 Gorged Feline Lane', 'Fakeville', 'Highlands', 654321, -2);

--ADD SUPPLIERS
INSERT INTO northwind_enhanceddb.SUPPLIERS VALUES(-1, 'Secret Society Inc', 'Jane Doe-Smith', 'Owner/Operator', '681 Street Road', 'Testville', 'Seaside', 465321, -1, '+22 (12-55) 521-454', '+22 (12-55) 521-454', 'example.com');
INSERT INTO northwind_enhanceddb.SUPPLIERS VALUES(-2, 'Sweaty Shop Company', 'Joe Smith-Doe', 'Frontline Manager', '861 Avenue Street', 'Fakeville', 'Highlands', 213456, -2, '+3 (18-55) 521-544', '+3 (189-55) 521-545', 'anotherexample.com');

--ADD CATEGORIES
INSERT INTO northwind_enhanceddb.CATEGORIES VALUES(-1, 'Medical Suppl', 'Medical supplies for use with Humans.');
INSERT INTO northwind_enhanceddb.CATEGORIES VALUES(-2, 'Pet Food', 'Food for intended for animals only.');

--ADD PRODUCTS
INSERT INTO northwind_enhanceddb.PRODUCTS VALUES(-1, 'Bulk Bandaid', -2, -1, '10000 Box x 100 unit', 10.74, 0);
INSERT INTO northwind_enhanceddb.PRODUCTS VALUES(-2, 'Mystery Meat', -1, -2, '20 Drum x 1.2 Gg', 1400.14, 1);

--ADD WAREHOUSESTOCK
INSERT INTO northwind_enhanceddb.WAREHOUSESTOCK VALUES (-2, -2, 1, 0, 0);
INSERT INTO northwind_enhanceddb.WAREHOUSESTOCK VALUES (-1, -1, 1, 0, 0);

--ADD REGION
INSERT INTO northwind_enhanceddb.REGION VALUES (-1, 'Southnorthern');
INSERT INTO northwind_enhanceddb.REGION VALUES (-2, 'Eastwestern');

--ADD TERRITORIES
INSERT INTO northwind_enhanceddb.TERRITORIES VALUES (-1, 'Antartica', -1);
INSERT INTO northwind_enhanceddb.TERRITORIES VALUES (-2, 'Magnetic North Pole', -2);

--ADD EMPLOYEES
INSERT INTO northwind_enhanceddb.EMPLOYEES VALUES (-1, 'Loblaw', 'Bob', 'Marketing Manager', 'Dr.', 'Dr. Bob Loblaw', TO_DATE('01/01/1970', 'mm/dd/yyyy'), TO_DATE('11/05/2001', 'mm/dd/yyyy'), '475 Road Avenue', 'Mountian View', 'Mountside', 987654, -1, '+1 555 123 4567', 6741, 'No notes on file for this employee. BOB', 1);
INSERT INTO northwind_enhanceddb.EMPLOYEES VALUES (-2, 'O''Donnell', 'Shannon', 'Sales Team Lead', 'Ms.', 'Ms. Shannon O''Donnell', TO_DATE('12/31/1970', 'mm/dd/yyyy'), TO_DATE('11/05/2001', 'mm/dd/yyyy'), '475 Avenue Way', 'New Sydney', 'Highlands', 875694, -2, '+1 555 321 4567', 6741, 'No notes on file for this employee. SHANNON', -1);

--ADD EMPLOYEETERRITORIES
INSERT INTO northwind_enhanceddb.EMPLOYEETERRITORIES VALUES(-1, -1);
INSERT INTO northwind_enhanceddb.EMPLOYEETERRITORIES VALUES(-1, -2);
INSERT INTO northwind_enhanceddb.EMPLOYEETERRITORIES VALUES(-2, -1);
INSERT INTO northwind_enhanceddb.EMPLOYEETERRITORIES VALUES(-2, -2);

--ADD CUSTOMERS
INSERT INTO northwind_enhanceddb.CUSTOMERS VALUES (-1, 'SLOCO', 'SLOW COMPANY', 'Joe Smith', '123 Rou Road', 'Riverside', 'Oceanside', '521498', -1, '00-12-34-65-781', '00-12-34-65-782');
INSERT INTO northwind_enhanceddb.CUSTOMERS VALUES (-2, 'AVGCO', 'Average COMPANY', 'Jane Smith', '132 Road Boulevard', 'Bayside', 'Maritimes', '521894', -2, '+ 22 1234 5678', '+23 1234 5678');

--ADD SHIPPERS
INSERT INTO northwind_enhanceddb.SHIPPERS VALUES (-1, 'HLD', '+1 (555) 555-1234');
INSERT INTO northwind_enhanceddb.SHIPPERS VALUES (-2, 'MedDev', '+1 (555) 555-4321');

--ADD ORDERS
INSERT INTO northwind_enhanceddb.ORDERS VALUES (-1, -1, -1, TO_DATE('04/11/2016', 'mm/dd/yyyy'), TO_DATE('04/18/2016', 'mm/dd/yyyy'), TO_DATE('04/12/2016', 'mm/dd/yyyy'), -1, 245.12, 'SLOW COMPANY', '123 Rou Road', 'Riverside', 'Oceanside', '521498', -1);
INSERT INTO northwind_enhanceddb.ORDERS VALUES (-2, -1, -1, TO_DATE('04/11/2016', 'mm/dd/yyyy'), TO_DATE('04/18/2016', 'mm/dd/yyyy'), TO_DATE('04/12/2016', 'mm/dd/yyyy'), -1, 547.41, 'Average Company', '132 Road Boulevard', 'Bayside', 'Maritimes', '521894', -2);

--ADD ORDER_DETAILS
INSERT INTO northwind_enhanceddb.ORDER_DETAILS VALUES (-1, -2, 10.74, 12, 0);
INSERT INTO northwind_enhanceddb.ORDER_DETAILS VALUES (-1, -1, 1400.14, 12, 40.00);
INSERT INTO northwind_enhanceddb.ORDER_DETAILS VALUES (-2, -1, 10.74, 1200, 100.00);
INSERT INTO northwind_enhanceddb.ORDER_DETAILS VALUES (-2, -2, 1400.14, 1, 200.00);


--TEST DATA REMOVAL

--DELETE ORDER_DETAILS
DELETE FROM NORTHWIND_ENHANCEDDB.ORDER_DETAILS
WHERE ORDERID < 0;

--DELETE ORDERS
DELETE FROM NORTHWIND_ENHANCEDDB.ORDERS
WHERE ORDERID < 0;

--DELETE SHIPPERS
DELETE FROM NORTHWIND_ENHANCEDDB.SHIPPERS
WHERE SHIPPERID < 0;

--DELETE CUSTOMERS
DELETE FROM NORTHWIND_ENHANCEDDB.CUSTOMERS
WHERE CUSTOMERID < 0;

--DELETE EMPLOYEETERRITORIES
DELETE FROM NORTHWIND_ENHANCEDDB.EMPLOYEETERRITORIES
WHERE EMPLOYEEID < 0;

--DELETE EMPLOYEES
DELETE FROM NORTHWIND_ENHANCEDDB.EMPLOYEES
WHERE EMPLOYEEID < 0;

--DELETE TERRITORIES
DELETE FROM NORTHWIND_ENHANCEDDB.TERRITORIES
WHERE TERRITORYID < 0;

--DELETE REGION
DELETE FROM NORTHWIND_ENHANCEDDB.REGION
WHERE REGIONID < 0;

--DELETE WAREHOUSESTOCK
DELETE FROM NORTHWIND_ENHANCEDDB.WAREHOUSESTOCK
WHERE PRODUCTID < 0;

--DELETE PRODUCTS
DELETE FROM NORTHWIND_ENHANCEDDB.PRODUCTS
WHERE PRODUCTID < 0;

--DELETE CATEGORIES
DELETE FROM NORTHWIND_ENHANCEDDB.CATEGORIES
WHERE CATEGORYID < 0;

--DELETE SUPPLIERS
DELETE FROM NORTHWIND_ENHANCEDDB.SUPPLIERS
WHERE SUPPLIERID < 0;

--DELETE WAREHOUSES
DELETE FROM NORTHWIND_ENHANCEDDB.WAREHOUSES
WHERE WAREHOUSEID < 0;

--DELETE COUNTRIES
DELETE FROM COUNTRIES
WHERE COUNTRYID < 0;

